@@ -7,6 +7,7 @@ Simple key/value store using Django models
 def get(key):
     """Return the value of a key"""
     from plinth.models import KVStore
+
     # pylint: disable-msg=E1101
     return KVStore.objects.get(pk=key).value
 
@@ -26,7 +27,11 @@ def set(key, value):  # pylint: disable-msg=W0622
     store.save()
 
 
-def delete(key):
+def delete(key, ignore_missing=False):
     """Delete a key"""
     from plinth.models import KVStore
-    return KVStore.objects.get(key=key).delete()
+    try:
+        return KVStore.objects.get(key=key).delete()
+    except KVStore.DoesNotExist:
+        if not ignore_missing:
+            raise
