@@ -59,7 +59,9 @@ class CoturnApp(app_module.App):
                               parent_url_name='apps')
         self.add(menu_item)
 
-        packages = Packages('packages-coturn', ['coturn'])
+        # Include sqlite3 to prevent removal of coturn from removal of
+        # roundcube.
+        packages = Packages('packages-coturn', ['coturn', 'sqlite3'])
         self.add(packages)
 
         firewall = Firewall('firewall-coturn', info.name,
@@ -112,6 +114,11 @@ class CoturnApp(app_module.App):
         self.get_component('letsencrypt-coturn').setup_certificates()
         notify_configuration_change()
 
+    def uninstall(self):
+        """De-configure and uninstall the app."""
+        super().uninstall()
+        privileged.uninstall()
+
 
 def get_available_domains():
     """Return an iterator with all domains able to have a certificate."""
