@@ -1,22 +1,21 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure a Deluge web client.
-"""
+"""FreedomBox app to configure a Deluge web client."""
 
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import frontpage, menu
+from plinth.config import DropinConfigs
 from plinth.daemon import Daemon
 from plinth.modules.apache.components import Webserver
 from plinth.modules.backups.components import BackupRestore
-from plinth.modules.firewall.components import Firewall
+from plinth.modules.firewall.components import (Firewall,
+                                                FirewallLocalProtection)
 from plinth.modules.users import add_user_to_share_group
 from plinth.modules.users.components import UsersAndGroups
 from plinth.package import Packages
 
-from . import manifest
+from . import manifest, privileged
 
 _description = [
     _('Deluge is a BitTorrent client that features a Web UI.'),
@@ -32,7 +31,7 @@ class DelugeApp(app_module.App):
 
     app_id = 'deluge'
 
-    _version = 6
+    _version = 8
 
     def __init__(self):
         """Create components for the app."""
@@ -67,10 +66,19 @@ class DelugeApp(app_module.App):
         packages = Packages('packages-deluge', ['deluged', 'deluge-web'])
         self.add(packages)
 
+        dropin_configs = DropinConfigs(
+            'dropin-configs-deluge',
+            ['/etc/apache2/conf-available/deluge-plinth.conf'])
+        self.add(dropin_configs)
+
         firewall = Firewall('firewall-deluge', info.name,
                             ports=['http', 'https'], is_external=True)
         self.add(firewall)
 
+        firewall_local_protection = FirewallLocalProtection(
+            'firewall-local-protection-deluge', ['8112'])
+        self.add(firewall_local_protection)
+
         webserver = Webserver('webserver-deluge', 'deluge-plinth',
                               urls=['https://{host}/deluge'])
         self.add(webserver)
@@ -96,5 +104,11 @@ class DelugeApp(app_module.App):
         """Install and configure the app."""
         super().setup(old_version)
         add_user_to_share_group(SYSTEM_USER)
-        actions.superuser_run('deluge', ['setup'])
-        self.enable()
+        privileged.setup()
+        if not old_version:
+            self.enable()
+
+    def uninstall(self):
+        """De-configure and uninstall the app."""
+        super().uninstall()
+        privileged.uninstall()
