@@ -1,14 +1,12 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure a firewall.
-"""
+"""FreedomBox app to configure a firewall."""
 
 import contextlib
 import logging
 
+from django.utils.translation import gettext
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import cfg, menu
 from plinth.daemon import Daemon
@@ -16,7 +14,7 @@ from plinth.modules.backups.components import BackupRestore
 from plinth.package import Packages, install
 from plinth.utils import Version, format_lazy, import_from_gi
 
-from . import manifest
+from . import manifest, privileged
 
 gio = import_from_gi('Gio', '2.0')
 glib = import_from_gi('GLib', '2.0')
@@ -37,10 +35,12 @@ _DBUS_NAME = 'org.fedoraproject.FirewallD1'
 _FIREWALLD_OBJECT = '/org/fedoraproject/FirewallD1'
 _FIREWALLD_INTERFACE = 'org.fedoraproject.FirewallD1'
 _ZONE_INTERFACE = 'org.fedoraproject.FirewallD1.zone'
+_DIRECT_INTERFACE = 'org.fedoraproject.FirewallD1.direct'
 _CONFIG_OBJECT = '/org/fedoraproject/FirewallD1/config'
 _CONFIG_INTERFACE = 'org.fedoraproject.FirewallD1.config'
 _CONFIG_SERVICE_INTERFACE = 'org.fedoraproject.FirewallD1.config.service'
 _CONFIG_ZONE_INTERFACE = 'org.fedoraproject.FirewallD1.config.zone'
+_CONFIG_DIRECT_INTERFACE = 'org.fedoraproject.FirewallD1.config.direct'
 
 
 class FirewallApp(app_module.App):
@@ -48,7 +48,7 @@ class FirewallApp(app_module.App):
 
     app_id = 'firewall'
 
-    _version = 2
+    _version = 3
 
     can_be_disabled = False
 
@@ -86,21 +86,28 @@ class FirewallApp(app_module.App):
         if 'firewalld' not in packages:
             return False
 
-        # firewalld 0.6.x -> 0.7.x, 0.6.x -> 0.8.x, 0.7.x -> 0.8.x, 0.9.x ->
-        # 1.0.x
+        # Allow upgrade from any version to any version below 2.0
         package = packages['firewalld']
-        if Version(package['current_version']) >= Version('1.0') or \
-           Version(package['new_version']) < Version('0.7'):
+        if Version(package['new_version']) > Version('2~'):
             return False
 
         install(['firewalld'], force_configuration='new')
         _run_setup()
         return True
 
+    def diagnose(self):
+        """Run diagnostics and return the results."""
+        results = super().diagnose()
+        config = privileged.get_config()
+        results.append(_diagnose_default_zone(config))
+        results.append(_diagnose_firewall_backend(config))
+        results.append(_diagnose_direct_passthroughs(config))
+        return results
+
 
 def _run_setup():
     """Run firewalld setup."""
-    _run(['setup'], superuser=True)
+    privileged.setup()
     add_service('http', 'external')
     add_service('http', 'internal')
     add_service('https', 'external')
@@ -173,17 +180,8 @@ def try_with_reload(operation):
         operation()
 
 
-def get_enabled_status():
-    """Return whether firewall is enabled"""
-    output = _run(['get-status'], superuser=True)
-    if not output:
-        return False
-    else:
-        return output.split()[0] == 'running'
-
-
 def get_enabled_services(zone):
-    """Return the status of various services currently enabled"""
+    """Return the status of various services currently enabled."""
     with ignore_dbus_error(dbus_error='ServiceUnknown'):
         zone_proxy = _get_dbus_proxy(_FIREWALLD_OBJECT, _ZONE_INTERFACE)
         return zone_proxy.getServices('(s)', zone)
@@ -192,7 +190,7 @@ def get_enabled_services(zone):
 
 
 def get_port_details(service_port):
-    """Return the port types and numbers for a service port"""
+    """Return the port types and numbers for a service port."""
     try:
         return _port_details[service_port]
     except KeyError:
@@ -217,7 +215,7 @@ def get_interfaces(zone):
 
 
 def add_service(port, zone):
-    """Enable a service in firewall"""
+    """Enable a service in firewall."""
     with ignore_dbus_error(dbus_error='ServiceUnknown'):
         zone_proxy = _get_dbus_proxy(_FIREWALLD_OBJECT, _ZONE_INTERFACE)
         with ignore_dbus_error(service_error='ALREADY_ENABLED'):
@@ -231,7 +229,7 @@ def add_service(port, zone):
 
 
 def remove_service(port, zone):
-    """Remove a service in firewall"""
+    """Remove a service in firewall."""
     with ignore_dbus_error(dbus_error='ServiceUnknown'):
         zone_proxy = _get_dbus_proxy(_FIREWALLD_OBJECT, _ZONE_INTERFACE)
         with ignore_dbus_error(service_error='NOT_ENABLED'):
@@ -244,11 +242,48 @@ def remove_service(port, zone):
             config_zone.removeService('(s)', port)
 
 
-def _run(arguments, superuser=False):
-    """Run an given command and raise exception if there was an error"""
-    command = 'firewall'
+def add_passthrough(ipv, *args):
+    """Add a direct rule with passthrough to ip(6)tables."""
+    direct_proxy = _get_dbus_proxy(_FIREWALLD_OBJECT, _DIRECT_INTERFACE)
+    if not direct_proxy.queryPassthrough('(sas)', ipv, args):
+        direct_proxy.addPassthrough('(sas)', ipv, args)
 
-    if superuser:
-        return actions.superuser_run(command, arguments)
-    else:
-        return actions.run(command, arguments)
+    config_direct = _get_dbus_proxy(_CONFIG_OBJECT, _CONFIG_DIRECT_INTERFACE)
+    if not config_direct.queryPassthrough('(sas)', ipv, args):
+        config_direct.addPassthrough('(sas)', ipv, args)
+
+
+def remove_passthrough(ipv, *args):
+    """Add a direct rule with passthrough to ip(6)tables."""
+    direct_proxy = _get_dbus_proxy(_FIREWALLD_OBJECT, _DIRECT_INTERFACE)
+    if direct_proxy.queryPassthrough('(sas)', ipv, args):
+        direct_proxy.removePassthrough('(sas)', ipv, args)
+
+    config_direct = _get_dbus_proxy(_CONFIG_OBJECT, _CONFIG_DIRECT_INTERFACE)
+    if config_direct.queryPassthrough('(sas)', ipv, args):
+        config_direct.removePassthrough('(sas)', ipv, args)
+
+
+def _diagnose_default_zone(config):
+    """Diagnose whether the default zone is external."""
+    testname = gettext('Default zone is external')
+    result = 'passed' if config['default_zone'] == 'external' else 'failed'
+    return [testname, result]
+
+
+def _diagnose_firewall_backend(config):
+    """Diagnose whether the firewall backend is nftables."""
+    testname = gettext('Firewall backend is nftables')
+    result = 'passed' if config['backend'] == 'nftables' else 'failed'
+    return [testname, result]
+
+
+def _diagnose_direct_passthroughs(config):
+    """Diagnose direct passthroughs for local service protection.
+
+    Currently, we just check that the number of passthroughs is at least 12,
+    which are the number that are added by firewall's setup.
+    """
+    testname = gettext('Direct passthrough rules exist')
+    result = 'passed' if len(config['passthroughs']) >= 12 else 'failed'
+    return [testname, result]
