@@ -1,22 +1,21 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure I2P.
-"""
+"""FreedomBox app to configure I2P."""
 
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import frontpage, menu
+from plinth.config import DropinConfigs
 from plinth.daemon import Daemon
 from plinth.modules.apache.components import Webserver
 from plinth.modules.backups.components import BackupRestore
-from plinth.modules.firewall.components import Firewall
+from plinth.modules.firewall.components import (Firewall,
+                                                FirewallLocalProtection)
 from plinth.modules.i2p.resources import FAVORITES
 from plinth.modules.users.components import UsersAndGroups
 from plinth.package import Packages
 
-from . import manifest
+from . import manifest, privileged
 
 _description = [
     _('The Invisible Internet Project is an anonymous network layer intended '
@@ -41,7 +40,7 @@ class I2PApp(app_module.App):
 
     app_id = 'i2p'
 
-    _version = 1
+    _version = 3
 
     def __init__(self):
         """Create components for the app."""
@@ -49,12 +48,11 @@ class I2PApp(app_module.App):
 
         groups = {'i2p': _('Manage I2P application')}
 
-        info = app_module.Info(
-            app_id=self.app_id, version=self._version, name=_('I2P'),
-            icon_filename='i2p', short_description=_('Anonymity Network'),
-            description=_description, manual_page='I2P',
-            clients=manifest.clients,
-            donation_url='https://geti2p.net/en/get-involved/donate')
+        info = app_module.Info(app_id=self.app_id, version=self._version,
+                               name=_('I2P'), icon_filename='i2p',
+                               short_description=_('Anonymity Network'),
+                               description=_description, manual_page='I2P',
+                               clients=manifest.clients)
         self.add(info)
 
         menu_item = menu.Menu('menu-i2p', info.name, info.short_description,
@@ -73,6 +71,11 @@ class I2PApp(app_module.App):
         packages = Packages('packages-i2p', ['i2p'])
         self.add(packages)
 
+        dropin_configs = DropinConfigs('dropin-configs-i2p', [
+            '/etc/apache2/conf-available/i2p-freedombox.conf',
+        ])
+        self.add(dropin_configs)
+
         firewall = Firewall('firewall-i2p-web', info.name,
                             ports=['http', 'https'], is_external=True)
         self.add(firewall)
@@ -82,6 +85,10 @@ class I2PApp(app_module.App):
                             is_external=False)
         self.add(firewall)
 
+        firewall_local_protection = FirewallLocalProtection(
+            'firewall-local-protection-i2p', ['7657'])
+        self.add(firewall_local_protection)
+
         webserver = Webserver('webserver-i2p', 'i2p-freedombox',
                               urls=['https://{host}/i2p/'])
         self.add(webserver)
@@ -100,28 +107,16 @@ class I2PApp(app_module.App):
         """Install and configure the app."""
         super().setup(old_version)
 
-        self.disable()
-        # Add favorites to the configuration
-        for fav in FAVORITES:
-            args = [
-                'add-favorite',
-                '--name',
-                fav.get('name'),
-                '--url',
-                fav.get('url'),
-            ]
-            if 'icon' in fav:
-                args.extend(['--icon', fav.get('icon')])
+        if not old_version:
+            self.disable()
+            # Add favorites to the configuration
+            for fav in FAVORITES:
+                privileged.add_favorite(fav['name'], fav['url'],
+                                        fav.get('description'),
+                                        fav.get('icon'))
 
-            if 'description' in fav:
-                args.extend(['--description', fav.get('description')])
+            # Tunnels to all interfaces
+            for tunnel in tunnels_to_manage:
+                privileged.set_tunnel_property(tunnel, 'interface', '0.0.0.0')
 
-            actions.superuser_run('i2p', args)
-
-        # Tunnels to all interfaces
-        for tunnel in tunnels_to_manage:
-            actions.superuser_run('i2p', [
-                'set-tunnel-property', '--name', tunnel, '--property',
-                'interface', '--value', '0.0.0.0'
-            ])
-        self.enable()
+            self.enable()
