@@ -1,14 +1,12 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure ikiwiki.
-"""
+"""FreedomBox app to configure ikiwiki."""
 
 from django.urls import reverse_lazy
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import cfg, frontpage, menu
+from plinth.config import DropinConfigs
 from plinth.modules.apache.components import Webserver
 from plinth.modules.backups.components import BackupRestore
 from plinth.modules.firewall.components import Firewall
@@ -16,7 +14,7 @@ from plinth.modules.users.components import UsersAndGroups
 from plinth.package import Packages
 from plinth.utils import format_lazy
 
-from . import manifest
+from . import manifest, privileged
 
 _description = [
     _('ikiwiki is a simple wiki and blog application. It supports '
@@ -37,7 +35,7 @@ class IkiwikiApp(app_module.App):
 
     app_id = 'ikiwiki'
 
-    _version = 1
+    _version = 2
 
     def __init__(self):
         """Create components for the app."""
@@ -64,6 +62,13 @@ class IkiwikiApp(app_module.App):
         ])
         self.add(packages)
 
+        dropin_configs = DropinConfigs('dropin-configs-ikiwiki', [
+            '/etc/ikiwiki/plinth-blog.setup',
+            '/etc/ikiwiki/plinth-wiki.setup',
+            '/etc/apache2/conf-available/ikiwiki-plinth.conf',
+        ])
+        self.add(dropin_configs)
+
         firewall = Firewall('firewall-ikiwiki', info.name,
                             ports=['http', 'https'], is_external=True)
         self.add(firewall)
@@ -77,8 +82,8 @@ class IkiwikiApp(app_module.App):
                                           groups=groups)
         self.add(users_and_groups)
 
-        backup_restore = BackupRestore('backup-restore-ikiwiki',
-                                       **manifest.backup)
+        backup_restore = IkiwikiBackupRestore('backup-restore-ikiwiki',
+                                              **manifest.backup)
         self.add(backup_restore)
 
     def post_init(self):
@@ -100,9 +105,8 @@ class IkiwikiApp(app_module.App):
         component.remove()  # Remove from global list.
 
     def refresh_sites(self):
-        """Refresh blog and wiki list"""
-        sites = actions.run('ikiwiki', ['get-sites']).split('\n')
-        sites = [name.split(' ', 1) for name in sites if name != '']
+        """Refresh blog and wiki list."""
+        sites = privileged.get_sites()
 
         for site in sites:
             if not 'shortcut-ikiwiki-' + site[0] in self.components:
@@ -113,5 +117,22 @@ class IkiwikiApp(app_module.App):
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        actions.superuser_run('ikiwiki', ['setup'])
-        self.enable()
+        privileged.setup()
+        if not old_version:
+            self.enable()
+
+    def uninstall(self):
+        """De-configure and uninstall the app."""
+        super().uninstall()
+        privileged.uninstall()
+
+
+class IkiwikiBackupRestore(BackupRestore):
+    """Component to handle Ikiwiki restore"""
+
+    def restore_post(self, packet):
+        """Re-run setup for each wiki after restore."""
+        super().restore_post(packet)
+        sites = privileged.get_sites()
+        for site in sites:
+            privileged.setup_site(site[0])
