@@ -1,7 +1,5 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure XMPP web client/jsxc.
-"""
+"""FreedomBox app to configure XMPP web client/jsxc."""
 
 import logging
 
@@ -32,8 +30,6 @@ class JSXCApp(app_module.App):
 
     _version = 1
 
-    can_be_disabled = False
-
     def __init__(self):
         """Create components for the app."""
         super().__init__()
@@ -50,6 +46,9 @@ class JSXCApp(app_module.App):
                               parent_url_name='apps')
         self.add(menu_item)
 
+        enable_state = app_module.EnableState('enable-state-jsxc')
+        self.add(enable_state)
+
         shortcut = frontpage.Shortcut('shortcut-jsxc', name=info.name,
                                       short_description=info.short_description,
                                       icon=info.icon_filename,
@@ -81,4 +80,5 @@ class JSXCApp(app_module.App):
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        self.enable()
+        if not old_version:
+            self.enable()
