@@ -1,7 +1,5 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure matrix-synapse server.
-"""
+"""FreedomBox app to configure matrix-synapse server."""
 
 import logging
 import os
@@ -11,9 +9,9 @@ from django.urls import reverse_lazy
 from django.utils.translation import gettext_lazy as _
 from ruamel.yaml.util import load_yaml_guess_indent
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import frontpage, menu
+from plinth.config import DropinConfigs
 from plinth.daemon import Daemon
 from plinth.modules.apache.components import Webserver
 from plinth.modules.backups.components import BackupRestore
@@ -23,7 +21,7 @@ from plinth.modules.letsencrypt.components import LetsEncrypt
 from plinth.package import Packages, install
 from plinth.utils import format_lazy, is_non_empty_file
 
-from . import manifest
+from . import manifest, privileged
 
 _description = [
     _('<a href="https://matrix.org/docs/guides/faq.html">Matrix</a> is an new '
@@ -41,23 +39,13 @@ _description = [
 
 logger = logging.getLogger(__name__)
 
-CONF_DIR = "/etc/matrix-synapse/conf.d/"
-
-ORIG_CONF_PATH = '/etc/matrix-synapse/homeserver.yaml'
-SERVER_NAME_PATH = CONF_DIR + 'server_name.yaml'
-STATIC_CONF_PATH = CONF_DIR + 'freedombox-static.yaml'
-LISTENERS_CONF_PATH = CONF_DIR + 'freedombox-listeners.yaml'
-REGISTRATION_CONF_PATH = CONF_DIR + 'freedombox-registration.yaml'
-TURN_CONF_PATH = CONF_DIR + 'freedombox-turn.yaml'
-OVERRIDDEN_TURN_CONF_PATH = CONF_DIR + 'turn.yaml'
-
 
 class MatrixSynapseApp(app_module.App):
     """FreedomBox app for Matrix Synapse."""
 
     app_id = 'matrixsynapse'
 
-    _version = 7
+    _version = 9
 
     def __init__(self):
         """Create components for the app."""
@@ -83,10 +71,20 @@ class MatrixSynapseApp(app_module.App):
             clients=info.clients, login_required=True)
         self.add(shortcut)
 
-        packages = Packages('packages-matrixsynapse',
-                            ['matrix-synapse', 'matrix-synapse-ldap3'])
+        # Include python3-psycopg2 to prevent accidental uninstall
+        # (see issue #2298).
+        packages = Packages(
+            'packages-matrixsynapse',
+            ['matrix-synapse', 'matrix-synapse-ldap3', 'python3-psycopg2'])
         self.add(packages)
 
+        dropin_configs = DropinConfigs('dropin-configs-matrixsynapse', [
+            '/etc/apache2/conf-available/matrix-synapse-plinth.conf',
+            '/etc/fail2ban/jail.d/matrix-auth-freedombox.conf',
+            '/etc/fail2ban/filter.d/matrix-auth-freedombox.conf',
+        ])
+        self.add(dropin_configs)
+
         firewall = Firewall('firewall-matrixsynapse', info.name,
                             ports=['matrix-synapse-plinth'], is_external=True)
         self.add(firewall)
@@ -122,7 +120,10 @@ class MatrixSynapseApp(app_module.App):
         if old_version and old_version < 6:
             upgrade()
         else:
-            actions.superuser_run('matrixsynapse', ['post-install'])
+            privileged.post_install()
+
+        if old_version and old_version <= 7:
+            privileged.fix_public_registrations()
 
         if not old_version:
             self.enable()
@@ -133,6 +134,11 @@ class MatrixSynapseApp(app_module.App):
         config = self.get_component('turn-matrixsynapse').get_configuration()
         update_turn_configuration(config, force=True)
 
+    def uninstall(self):
+        """De-configure and uninstall the app."""
+        super().uninstall()
+        privileged.uninstall()
+
 
 class MatrixSynapseTurnConsumer(TurnConsumer):
     """Component to manage Coturn configuration for Matrix Synapse."""
@@ -144,14 +150,12 @@ class MatrixSynapseTurnConsumer(TurnConsumer):
 
 def upgrade():
     """Upgrade matrix-synapse configuration to avoid conffile prompt."""
-    public_registration_status = get_public_registration_status()
-    actions.superuser_run('matrixsynapse', ['move-old-conf'])
+    config = privileged.get_config()
+    privileged.move_old_conf()
     install(['matrix-synapse'], force_configuration='new', reinstall=True,
             force_missing_configuration=True)
-    actions.superuser_run('matrixsynapse', ['post-install'])
-    if public_registration_status:
-        actions.superuser_run('matrixsynapse',
-                              ['public-registration', 'enable'])
+    privileged.post_install()
+    privileged.set_config(**config)
 
 
 def setup_domain(domain_name):
@@ -159,13 +163,12 @@ def setup_domain(domain_name):
     app = app_module.App.get('matrixsynapse')
     app.get_component('letsencrypt-matrixsynapse').setup_certificates(
         [domain_name])
-    actions.superuser_run('matrixsynapse',
-                          ['setup', '--domain-name', domain_name])
+    privileged.setup(domain_name)
 
 
 def is_setup():
     """Return whether the Matrix Synapse server is setup."""
-    return os.path.exists(SERVER_NAME_PATH)
+    return os.path.exists(privileged.SERVER_NAME_PATH)
 
 
 def get_domains():
@@ -182,7 +185,7 @@ def get_configured_domain_name():
     if not is_setup():
         return None
 
-    with open(SERVER_NAME_PATH, encoding='utf-8') as config_file:
+    with open(privileged.SERVER_NAME_PATH, encoding='utf-8') as config_file:
         config, _, _ = load_yaml_guess_indent(config_file)
 
     return config['server_name']
@@ -190,8 +193,8 @@ def get_configured_domain_name():
 
 def get_turn_configuration() -> (List[str], str, bool):
     """Return TurnConfiguration if setup else empty."""
-    for file_path, managed in ((OVERRIDDEN_TURN_CONF_PATH, False),
-                               (TURN_CONF_PATH, True)):
+    for file_path, managed in ((privileged.OVERRIDDEN_TURN_CONF_PATH, False),
+                               (privileged.TURN_CONF_PATH, True)):
         if is_non_empty_file(file_path):
             with open(file_path, encoding='utf-8') as config_file:
                 config, _, _ = load_yaml_guess_indent(config_file)
@@ -202,13 +205,6 @@ def get_turn_configuration() -> (List[str], str, bool):
     return (TurnConfiguration(), True)
 
 
-def get_public_registration_status() -> bool:
-    """Return whether public registration is enabled."""
-    output = actions.superuser_run('matrixsynapse',
-                                   ['public-registration', 'status'])
-    return output.strip() == 'enabled'
-
-
 def get_certificate_status():
     """Return the status of certificate for the configured domain."""
     app = app_module.App.get('matrixsynapse')
@@ -226,7 +222,4 @@ def update_turn_configuration(config: TurnConfiguration, managed=True,
     if not force and app.needs_setup():
         return
 
-    params = ['configure-turn']
-    params += ['--managed'] if managed else []
-    actions.superuser_run('matrixsynapse', params,
-                          input=config.to_json().encode())
+    privileged.configure_turn(managed, config.to_json())
