@@ -1,7 +1,7 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-Views for the Matrix Synapse module.
-"""
+"""Views for the Matrix Synapse module."""
+
+import datetime
 
 from django.contrib import messages
 from django.shortcuts import redirect
@@ -9,19 +9,19 @@ from django.urls import reverse_lazy
 from django.utils.translation import gettext_lazy as _
 from django.views.generic import FormView
 
-from plinth import actions
 from plinth import app as app_module
 from plinth.forms import DomainSelectionForm
 from plinth.modules import matrixsynapse, names
 from plinth.modules.coturn.components import TurnConfiguration
 from plinth.views import AppView
 
-from . import get_public_registration_status, get_turn_configuration
+from . import get_turn_configuration, privileged
 from .forms import MatrixSynapseForm
 
 
 class SetupView(FormView):
     """Show matrix-synapse setup page."""
+
     template_name = 'matrix-synapse-pre-setup.html'
     form_class = DomainSelectionForm
     success_url = reverse_lazy('matrixsynapse:index')
@@ -46,6 +46,7 @@ class SetupView(FormView):
 
 class MatrixSynapseAppView(AppView):
     """Show matrix-synapse service page."""
+
     app_id = 'matrixsynapse'
     template_name = 'matrix-synapse.html'
     form_class = MatrixSynapseForm
@@ -62,29 +63,36 @@ class MatrixSynapseAppView(AppView):
         context = super().get_context_data(*args, **kwargs)
         context['domain_name'] = matrixsynapse.get_configured_domain_name()
         context['certificate_status'] = matrixsynapse.get_certificate_status()
+        context['config'] = privileged.get_config()
+        tokens = privileged.list_registration_tokens()
+        for token in tokens:
+            if token['expiry_time']:
+                date = datetime.datetime.utcfromtimestamp(
+                    token['expiry_time'] / 1000)
+                token['expiry_time'] = date
+
+        context['registration_tokens'] = tokens
         return context
 
     def get_initial(self):
         """Return the values to fill in the form."""
         initial = super().get_initial()
-        config, managed = get_turn_configuration()
+        turn_config, managed = get_turn_configuration()
+        config = privileged.get_config()
         initial.update({
-            'enable_public_registration': get_public_registration_status(),
-            'enable_managed_turn': managed,
-            'turn_uris': '\n'.join(config.uris),
-            'shared_secret': config.shared_secret
+            'enable_public_registration':
+                config['public_registration'],
+            'registration_verification':
+                config['registration_verification'] or 'disabled',
+            'enable_managed_turn':
+                managed,
+            'turn_uris':
+                '\n'.join(turn_config.uris),
+            'shared_secret':
+                turn_config.shared_secret
         })
         return initial
 
-    @staticmethod
-    def _handle_public_registrations(new_config):
-        if new_config['enable_public_registration']:
-            actions.superuser_run('matrixsynapse',
-                                  ['public-registration', 'enable'])
-        else:
-            actions.superuser_run('matrixsynapse',
-                                  ['public-registration', 'disable'])
-
     @staticmethod
     def _handle_turn_configuration(old_config, new_config):
         if not new_config['enable_managed_turn']:
@@ -113,9 +121,21 @@ class MatrixSynapseAppView(AppView):
             return old_config[prop] != new_config[prop]
 
         is_changed = False
-        if changed('enable_public_registration'):
-            self._handle_public_registrations(new_config)
-            is_changed = True
+        if (changed('enable_public_registration')
+                or changed('registration_verification')):
+            try:
+                privileged.set_config(
+                    public_registration=new_config[
+                        'enable_public_registration'],
+                    registration_verification=new_config[
+                        'registration_verification'])
+                is_changed = True
+            except ProcessLookupError:
+                # Matrix Synapse server is not running
+                messages.error(
+                    self.request,
+                    _('Registration configuration cannot be updated when app '
+                      'is disabled.'))
 
         if changed('enable_managed_turn') or changed('turn_uris') or \
            changed('shared_secret'):
