@@ -4,10 +4,11 @@ FreedomBox app to configure minidlna.
 """
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import frontpage, menu
+from plinth.config import DropinConfigs
 from plinth.daemon import Daemon
+from plinth.modules import firewall
 from plinth.modules.apache.components import Webserver
 from plinth.modules.backups.components import BackupRestore
 from plinth.modules.firewall.components import Firewall
@@ -15,7 +16,7 @@ from plinth.modules.users.components import UsersAndGroups
 from plinth.package import Packages, install
 from plinth.utils import Version
 
-from . import manifest
+from . import manifest, privileged
 
 _description = [
     _('MiniDLNA is a simple media server software, with the aim of being '
@@ -30,13 +31,14 @@ _description = [
 
 
 class MiniDLNAApp(app_module.App):
-    """Freedombox app managing miniDlna"""
+    """Freedombox app managing miniDlna."""
+
     app_id = 'minidlna'
 
-    _version = 2
+    _version = 5
 
     def __init__(self):
-        """Initialize the app components"""
+        """Initialize the app components."""
         super().__init__()
 
         groups = {'minidlna': _('Media streaming server')}
@@ -70,12 +72,17 @@ class MiniDLNAApp(app_module.App):
         packages = Packages('packages-minidlna', ['minidlna'])
         self.add(packages)
 
+        dropin_configs = DropinConfigs(
+            'dropin-configs-minidlna',
+            ['/etc/apache2/conf-available/minidlna-freedombox.conf'])
+        self.add(dropin_configs)
+
         firewall = Firewall('firewall-minidlna', info.name, ports=['minidlna'],
                             is_external=False)
         self.add(firewall)
 
         webserver = Webserver('webserver-minidlna', 'minidlna-freedombox',
-                              urls=['http://localhost:8200/'])
+                              urls=['https://{host}/_minidlna/'])
         self.add(webserver)
 
         daemon = Daemon('daemon-minidlna', 'minidlna')
@@ -92,7 +99,15 @@ class MiniDLNAApp(app_module.App):
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        actions.superuser_run('minidlna', ['setup'])
+        privileged.setup()
+        if old_version == 3:
+            # Version 3 of the app incorrectly declared port 8200 for firewall
+            # local protection.
+            firewall.remove_passthrough('ipv6', '-A', 'INPUT', '-p', 'tcp',
+                                        '--dport', '8200', '-j', 'REJECT')
+            firewall.remove_passthrough('ipv4', '-A', 'INPUT', '-p', 'tcp',
+                                        '--dport', '8200', '-j', 'REJECT')
+
         if not old_version:
             self.enable()
 
@@ -106,18 +121,8 @@ class MiniDLNAApp(app_module.App):
         if Version(package['new_version']) > Version('1.4~'):
             return False
 
-        media_dir = get_media_dir()
+        media_dir = privileged.get_media_dir()
         install(['minidlna'], force_configuration='new')
-        set_media_dir(media_dir)
+        privileged.set_media_dir(media_dir)
 
         return True
-
-
-def get_media_dir():
-    """Return the currently set media directory."""
-    return actions.superuser_run('minidlna', ['get-media-dir'])
-
-
-def set_media_dir(media_dir):
-    """Set the media directory from which files will be scanned for sharing."""
-    actions.superuser_run('minidlna', ['set-media-dir', '--dir', media_dir])
