@@ -1,18 +1,18 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to interface with network-manager.
-"""
+"""FreedomBox app to interface with network-manager."""
 
 import subprocess
 from logging import Logger
 
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import daemon, kvstore, menu, network
+from plinth.config import DropinConfigs
 from plinth.package import Packages
 
+from . import privileged
+
 first_boot_steps = [
     {
         'id': 'network_topology_wizard',
@@ -46,7 +46,9 @@ class NetworksApp(app_module.App):
 
     app_id = 'networks'
 
-    _version = 1
+    _version = 2
+
+    can_be_disabled = False
 
     def __init__(self):
         """Create components for the app."""
@@ -65,6 +67,11 @@ class NetworksApp(app_module.App):
         packages = Packages('packages-networks', ['network-manager', 'batctl'])
         self.add(packages)
 
+        dropin_configs = DropinConfigs('dropin-configs-networks', [
+            '/etc/NetworkManager/dispatcher.d/10-freedombox-batman',
+        ])
+        self.add(dropin_configs)
+
     def diagnose(self):
         """Run diagnostics and return the results."""
         results = super().diagnose()
@@ -81,7 +88,7 @@ class NetworksApp(app_module.App):
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        actions.superuser_run('networks')
+        privileged.setup()
         self.enable()
 
 
