@@ -1,14 +1,9 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure OpenVPN server.
-"""
-
-import os
+"""FreedomBox app to configure OpenVPN server."""
 
 from django.urls import reverse_lazy
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import cfg, frontpage, menu
 from plinth.daemon import Daemon
@@ -18,7 +13,7 @@ from plinth.modules.users.components import UsersAndGroups
 from plinth.package import Packages
 from plinth.utils import format_lazy
 
-from . import manifest
+from . import manifest, privileged
 
 _description = [
     format_lazy(
@@ -41,11 +36,6 @@ class OpenVPNApp(app_module.App):
 
     _version = 4
 
-    @property
-    def can_be_disabled(self):
-        """Return whether the app can be disabled."""
-        return is_setup()
-
     def __init__(self):
         """Create components for the app."""
         super().__init__()
@@ -96,32 +86,14 @@ class OpenVPNApp(app_module.App):
                                        **manifest.backup)
         self.add(backup_restore)
 
-    def is_enabled(self):
-        """Return whether all the leader components are enabled.
-
-        Return True when there are no leader components and OpenVPN setup
-        is done.
-        """
-        return super().is_enabled() and is_setup()
-
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        actions.superuser_run('openvpn', ['setup'])
-        self.enable()
+        privileged.setup()
+        if not old_version:
+            self.enable()
 
-
-def is_setup():
-    """Return whether the service is running."""
-    return actions.superuser_run('openvpn', ['is-setup']).strip() == 'true'
-
-
-def is_using_ecc():
-    """Return whether the service is using ECC."""
-    if os.path.exists(SERVER_CONFIGURATION_FILE):
-        with open(SERVER_CONFIGURATION_FILE, 'r',
-                  encoding='utf-8') as file_handle:
-            for line in file_handle:
-                if line.strip() == 'dh none':
-                    return True
-    return False
+    def uninstall(self):
+        """De-configure and uninstall the app."""
+        super().uninstall()
+        privileged.uninstall()
