@@ -8,9 +8,9 @@ import logging
 import augeas
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import cfg, frontpage, menu
+from plinth.config import DropinConfigs
 from plinth.modules.apache.components import Uwsgi, Webserver
 from plinth.modules.backups.components import BackupRestore
 from plinth.modules.firewall.components import Firewall
@@ -18,7 +18,7 @@ from plinth.modules.users.components import UsersAndGroups
 from plinth.package import Packages, install
 from plinth.utils import Version, format_lazy
 
-from . import manifest
+from . import manifest, privileged
 
 _description = [
     format_lazy(
@@ -43,7 +43,7 @@ class RadicaleApp(app_module.App):
 
     app_id = 'radicale'
 
-    _version = 2
+    _version = 3
 
     def __init__(self):
         """Create components for the app."""
@@ -72,6 +72,11 @@ class RadicaleApp(app_module.App):
         packages = Packages('packages-radicale', ['radicale'])
         self.add(packages)
 
+        dropin_configs = DropinConfigs('dropin-configs-radicale', [
+            '/etc/apache2/conf-available/radicale2-freedombox.conf',
+        ])
+        self.add(dropin_configs)
+
         firewall = Firewall('firewall-radicale', info.name,
                             ports=['http', 'https'], is_external=True)
         self.add(firewall)
@@ -93,13 +98,14 @@ class RadicaleApp(app_module.App):
 
     def enable(self):
         """Fix missing directories before enabling radicale."""
-        actions.superuser_run('radicale', ['fix-paths'])
+        privileged.fix_paths()
         super().enable()
 
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        self.enable()
+        if not old_version:
+            self.enable()
 
     def force_upgrade(self, packages):
         """Force upgrade radicale to resolve conffile prompt."""
@@ -113,8 +119,7 @@ class RadicaleApp(app_module.App):
 
         rights = get_rights_value()
         install(['radicale'], force_configuration='new')
-        actions.superuser_run('radicale',
-                              ['configure', '--rights_type', rights])
+        privileged.configure(rights)
 
         return True
 
