@@ -1,12 +1,9 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure Shadowsocks.
-"""
+"""FreedomBox app to configure Shadowsocks Client."""
 
 from django.urls import reverse_lazy
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import cfg, frontpage, menu
 from plinth.daemon import Daemon
@@ -15,12 +12,16 @@ from plinth.modules.firewall.components import Firewall
 from plinth.package import Packages
 from plinth.utils import format_lazy
 
-from . import manifest
+from . import manifest, privileged
 
 _description = [
-    _('Shadowsocks is a lightweight and secure SOCKS5 proxy, designed to '
-      'protect your Internet traffic. It can be used to bypass Internet '
-      'filtering and censorship.'),
+    _('Shadowsocks is a tool for securely forwarding network requests to a'
+      ' remote server. It consists of two parts: (1) a Shadowsocks server,'
+      ' and (2) a Shadowsocks client with a SOCKS5 proxy.'),
+    _('Shadowsocks can be used to bypass Internet filtering and '
+      'censorship. This requires that the Shadowsocks server is in a '
+      'location where it can freely access the Internet, without '
+      'filtering.'),
     format_lazy(
         _('Your {box_name} can run a Shadowsocks client, that can connect to '
           'a Shadowsocks server. It will also run a SOCKS5 proxy. Local '
@@ -33,7 +34,7 @@ _description = [
 
 
 class ShadowsocksApp(app_module.App):
-    """FreedomBox app for Shadowsocks."""
+    """FreedomBox app for Shadowsocks Client."""
 
     app_id = 'shadowsocks'
 
@@ -46,9 +47,9 @@ class ShadowsocksApp(app_module.App):
         super().__init__()
 
         info = app_module.Info(app_id=self.app_id, version=self._version,
-                               name=_('Shadowsocks'),
+                               name=_('Shadowsocks Client'),
                                icon_filename='shadowsocks',
-                               short_description=_('Socks5 Proxy'),
+                               short_description=_('Bypass Censorship'),
                                description=_description,
                                manual_page='Shadowsocks')
         self.add(info)
@@ -85,5 +86,11 @@ class ShadowsocksApp(app_module.App):
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        actions.superuser_run('shadowsocks', ['setup'])
-        self.enable()
+        privileged.setup()
+        if not old_version:
+            self.enable()
+
+    def uninstall(self):
+        """De-configure and uninstall the app."""
+        super().uninstall()
+        privileged.uninstall()
