@@ -1,15 +1,11 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure sharing.
-"""
-
-import json
+"""FreedomBox app to configure sharing."""
 
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import cfg, menu
+from plinth.config import DropinConfigs
 from plinth.modules.apache.components import Webserver
 from plinth.modules.backups.components import BackupRestore
 from plinth.utils import format_lazy
@@ -29,7 +25,7 @@ class SharingApp(app_module.App):
 
     app_id = 'sharing'
 
-    _version = 2
+    _version = 3
 
     def __init__(self):
         """Create components for the app."""
@@ -44,6 +40,11 @@ class SharingApp(app_module.App):
                               parent_url_name='apps')
         self.add(menu_item)
 
+        dropin_configs = DropinConfigs('dropin-configs-sharing', [
+            '/etc/apache2/includes/freedombox-sharing.conf',
+        ])
+        self.add(dropin_configs)
+
         webserver = Webserver('webserver-sharing', 'sharing-freedombox')
         self.add(webserver)
 
@@ -59,23 +60,10 @@ class SharingApp(app_module.App):
         """Install and configure the app."""
         super().setup(old_version)
         privileged.setup()
-        self.enable()
+        if not old_version:
+            self.enable()
 
-
-def list_shares():
-    """Return a list of shares."""
-    output = actions.superuser_run('sharing', ['list'])
-    return json.loads(output)['shares']
-
-
-def add_share(name, path, groups, is_public):
-    """Add a new share by called the action script."""
-    args = ['add', '--name', name, '--path', path, '--groups'] + groups
-    if is_public:
-        args.append('--is-public')
-    actions.superuser_run('sharing', args)
-
-
-def remove_share(name):
-    """Remove a share by calling the action script."""
-    actions.superuser_run('sharing', ['remove', '--name', name])
+    def uninstall(self):
+        """De-configure and uninstall the app."""
+        super().uninstall()
+        privileged.uninstall()
