@@ -1,7 +1,5 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app for OpenSSH server.
-"""
+"""FreedomBox app for OpenSSH server."""
 
 import pathlib
 import re
@@ -9,15 +7,16 @@ import subprocess
 
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import menu
+from plinth.config import DropinConfigs
 from plinth.daemon import Daemon
 from plinth.modules.backups.components import BackupRestore
 from plinth.modules.firewall.components import Firewall
+from plinth.modules.users.components import UsersAndGroups
 from plinth.package import Packages
 
-from . import manifest
+from . import manifest, privileged
 
 _description = [
     _('A Secure Shell server uses the secure shell protocol to accept '
@@ -32,7 +31,7 @@ class SSHApp(app_module.App):
 
     app_id = 'ssh'
 
-    _version = 1
+    _version = 4
 
     def __init__(self):
         """Create components for the app."""
@@ -52,6 +51,17 @@ class SSHApp(app_module.App):
         packages = Packages('packages-ssh', ['openssh-server'])
         self.add(packages)
 
+        dropin_configs = DropinConfigs('dropin-configs-ssh', [
+            '/etc/fail2ban/jail.d/ssh-freedombox.conf',
+        ])
+        self.add(dropin_configs)
+
+        dropin_configs = DropinConfigs('dropin-config-ssh-avahi', [
+            '/etc/avahi/services/sftp-ssh.service',
+            '/etc/avahi/services/ssh.service',
+        ], copy_only=True)
+        self.add(dropin_configs)
+
         firewall = Firewall('firewall-ssh', info.name, ports=['ssh'],
                             is_external=True)
         self.add(firewall)
@@ -59,14 +69,26 @@ class SSHApp(app_module.App):
         daemon = Daemon('daemon-ssh', 'ssh')
         self.add(daemon)
 
+        groups = {
+            'freedombox-ssh': _('Remotely login using Secure Shell (SSH)')
+        }
+        users_and_groups = UsersAndGroups('users-and-groups-ssh',
+                                          groups=groups)
+        self.add(users_and_groups)
+
         backup_restore = BackupRestore('backup-restore-ssh', **manifest.backup)
         self.add(backup_restore)
 
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        actions.superuser_run('ssh', ['setup'])
-        self.enable()
+        privileged.setup()
+        if not old_version:
+            self.enable()
+        elif old_version == 2 and privileged.are_users_restricted():
+            privileged.restrict_users(True)
+        elif old_version == 1:
+            privileged.restrict_users(True)
 
 
 def get_host_keys():
@@ -87,9 +109,3 @@ def get_host_keys():
                 host_keys.append(match.groupdict())
 
     return host_keys
-
-
-def is_password_authentication_disabled():
-    """Return if ssh password authentication is enabled."""
-    return actions.superuser_run('ssh',
-                                 ['get-password-config']).strip() == 'no'
