@@ -8,10 +8,12 @@ from django.utils.translation import gettext_lazy as _
 
 from plinth import app as app_module
 from plinth import cfg, frontpage, menu
+from plinth.config import DropinConfigs
 from plinth.daemon import Daemon
 from plinth.modules.apache.components import Webserver
 from plinth.modules.backups.components import BackupRestore
-from plinth.modules.firewall.components import Firewall
+from plinth.modules.firewall.components import (Firewall,
+                                                FirewallLocalProtection)
 from plinth.modules.users import add_user_to_share_group
 from plinth.modules.users.components import UsersAndGroups
 from plinth.package import Packages
@@ -32,6 +34,12 @@ _description = [
         _('It can be accessed by <a href="{users_url}">any user</a> on '
           '{box_name} belonging to the bit-torrent group.'),
         box_name=_(cfg.box_name), users_url=reverse_lazy('users:index')),
+    format_lazy(
+        _('In addition to the web interface, mobile and desktop apps can also '
+          'be used to remotely control Transmission on {box_name}. To '
+          'configure remote control apps, use the URL '
+          '<a href="/transmission-remote/rpc">/transmission-remote/rpc</a>.'),
+        box_name=_(cfg.box_name)),
     format_lazy(
         _('<a href="{samba_url}">Samba</a> shares can be set as the '
           'default download directory from the dropdown menu below.'),
@@ -50,7 +58,7 @@ class TransmissionApp(app_module.App):
 
     app_id = 'transmission'
 
-    _version = 4
+    _version = 7
 
     DAEMON = 'transmission-daemon'
 
@@ -86,13 +94,23 @@ class TransmissionApp(app_module.App):
         packages = Packages('packages-transmission', ['transmission-daemon'])
         self.add(packages)
 
+        dropin_configs = DropinConfigs('dropin-configs-transmission', [
+            '/etc/apache2/conf-available/transmission-plinth.conf',
+        ])
+        self.add(dropin_configs)
+
         firewall = Firewall('firewall-transmission', info.name,
                             ports=['http', 'https',
                                    'transmission-client'], is_external=True)
         self.add(firewall)
 
+        firewall_local_protection = FirewallLocalProtection(
+            'firewall-local-protection-transmission', ['9091'])
+        self.add(firewall_local_protection)
+
         webserver = Webserver('webserver-transmission', 'transmission-plinth',
-                              urls=['https://{host}/transmission'])
+                              urls=['https://{host}/transmission'],
+                              last_updated_version=6)
         self.add(webserver)
 
         daemon = Daemon(
@@ -126,4 +144,6 @@ class TransmissionApp(app_module.App):
         }
         privileged.merge_configuration(new_configuration)
         add_user_to_share_group(SYSTEM_USER, TransmissionApp.DAEMON)
-        self.enable()
+
+        if not old_version:
+            self.enable()
