@@ -1,9 +1,6 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app for upgrades.
-"""
+"""FreedomBox app for upgrades."""
 
-import json
 import logging
 import os
 import subprocess
@@ -13,14 +10,14 @@ from django.utils.translation import gettext_lazy as _
 from django.utils.translation import gettext_noop
 
 import plinth
-from plinth import actions
 from plinth import app as app_module
 from plinth import cfg, glib, kvstore, menu
+from plinth.config import DropinConfigs
 from plinth.daemon import RelatedDaemon
 from plinth.modules.backups.components import BackupRestore
 from plinth.package import Packages
 
-from . import manifest
+from . import manifest, privileged
 
 first_boot_steps = [
     {
@@ -48,10 +45,6 @@ BACKPORTS_REQUESTED_KEY = 'upgrades_backports_requested'
 
 DIST_UPGRADE_ENABLED_KEY = 'upgrades_dist_upgrade_enabled'
 
-SOURCES_LIST = '/etc/apt/sources.list'
-
-BACKPORTS_SOURCES_LIST = '/etc/apt/sources.list.d/freedombox2.list'
-
 logger = logging.getLogger(__name__)
 
 
@@ -60,7 +53,7 @@ class UpgradesApp(app_module.App):
 
     app_id = 'upgrades'
 
-    _version = 14
+    _version = 17
 
     can_be_disabled = False
 
@@ -82,6 +75,14 @@ class UpgradesApp(app_module.App):
                             ['unattended-upgrades', 'needrestart'])
         self.add(packages)
 
+        dropin_configs = DropinConfigs('dropin-configs-upgrades', [
+            '/etc/apt/apt.conf.d/20freedombox',
+            '/etc/apt/apt.conf.d/20freedombox-allow-release-info-change',
+            '/etc/apt/apt.conf.d/60unattended-upgrades',
+            '/etc/needrestart/conf.d/freedombox.conf',
+        ])
+        self.add(dropin_configs)
+
         daemon = RelatedDaemon('related-daemon-upgrades',
                                'freedombox-dist-upgrade')
         self.add(daemon)
@@ -140,11 +141,11 @@ class UpgradesApp(app_module.App):
 
         # Enable automatic upgrades but only on first install
         if not old_version and not cfg.develop:
-            actions.superuser_run('upgrades', ['enable-auto'])
+            privileged.enable_auto()
 
         # Update apt preferences whenever on first install and on version
         # increment.
-        actions.superuser_run('upgrades', ['setup'])
+        privileged.setup()
 
         # When upgrading from a version without first boot wizard for
         # backports, assume backports have been requested.
@@ -161,82 +162,65 @@ class UpgradesApp(app_module.App):
         setup_repositories(None)
 
 
-def is_enabled():
-    """Return whether the module is enabled."""
-    output = actions.run('upgrades', ['check-auto'])
-    return 'True' in output.split()
-
-
-def enable():
-    """Enable the module."""
-    actions.superuser_run('upgrades', ['enable-auto'])
-
-
-def disable():
-    """Disable the module."""
-    actions.superuser_run('upgrades', ['disable-auto'])
-
-
 def setup_repositories(_):
     """Setup apt repositories for backports."""
     if is_backports_requested():
-        command = ['activate-backports']
-        if cfg.develop:
-            command.append('--develop')
-
-        actions.superuser_run('upgrades', command)
+        privileged.activate_backports(cfg.develop)
 
 
 def check_dist_upgrade(_):
     """Check for upgrade to new stable release."""
-    from plinth.notification import Notification
     if is_dist_upgrade_enabled():
-        output = actions.superuser_run('upgrades', ['start-dist-upgrade'])
-        result = json.loads(output)
-        dist_upgrade_started = result['dist_upgrade_started']
-        reason = result['reason']
-        if 'found-previous' in reason:
-            logger.info(
-                'Found previous dist-upgrade. If it was interrupted, it will '
-                'be restarted.')
-        elif 'already-' in reason:
-            logger.info('Skip dist upgrade: System is already up-to-date.')
-        elif 'codename-not-found' in reason:
-            logger.warning('Skip dist upgrade: Codename not found in release '
-                           'file.')
-        elif 'upgrades-not-enabled' in reason:
-            logger.info('Skip dist upgrade: Automatic updates are not '
-                        'enabled.')
-        elif 'test-not-set' in reason:
-            logger.info('Skip dist upgrade: --test is not set.')
-        elif 'not-enough-free-space' in reason:
-            logger.warning('Skip dist upgrade: Not enough free space in /.')
-            title = gettext_noop('Could not start distribution update')
-            message = gettext_noop(
-                'There is not enough free space in the root partition to '
-                'start the distribution update. Please ensure at least 5 GB '
-                'is free. Distribution update will be retried after 24 hours,'
-                ' if enabled.')
-            Notification.update_or_create(
-                id='upgrades-dist-upgrade-free-space', app_id='upgrades',
-                severity='warning', title=title, message=message, actions=[{
-                    'type': 'dismiss'
-                }], group='admin')
-        elif 'started-dist-upgrade' in reason:
-            logger.info('Started dist upgrade.')
-            title = gettext_noop('Distribution update started')
-            message = gettext_noop(
-                'Started update to next stable release. This may take a long '
-                'time to complete.')
-            Notification.update_or_create(id='upgrades-dist-upgrade-started',
-                                          app_id='upgrades', severity='info',
-                                          title=title, message=message,
-                                          actions=[{
-                                              'type': 'dismiss'
-                                          }], group='admin')
-        else:
-            logger.warning('Unhandled result of start-dist-upgrade: %s, %s',
-                           dist_upgrade_started, reason)
+        try_start_dist_upgrade()
+
+
+def try_start_dist_upgrade(test=False):
+    """Try to start dist upgrade."""
+    from plinth.notification import Notification
+
+    result = privileged.start_dist_upgrade(test)
+    dist_upgrade_started = result['dist_upgrade_started']
+    reason = result['reason']
+    if 'found-previous' in reason:
+        logger.info(
+            'Found previous dist-upgrade. If it was interrupted, it will '
+            'be restarted.')
+    elif 'already-' in reason:
+        logger.info('Skip dist upgrade: System is already up-to-date.')
+    elif 'codename-not-found' in reason:
+        logger.warning('Skip dist upgrade: Codename not found in release '
+                       'file.')
+    elif 'upgrades-not-enabled' in reason:
+        logger.info('Skip dist upgrade: Automatic updates are not enabled.')
+    elif 'test-not-set' in reason:
+        logger.info('Skip dist upgrade: --test is not set.')
+    elif 'not-enough-free-space' in reason:
+        logger.warning('Skip dist upgrade: Not enough free space in /.')
+        title = gettext_noop('Could not start distribution update')
+        message = gettext_noop(
+            'There is not enough free space in the root partition to '
+            'start the distribution update. Please ensure at least 5 GB '
+            'is free. Distribution update will be retried after 24 hours,'
+            ' if enabled.')
+        Notification.update_or_create(id='upgrades-dist-upgrade-free-space',
+                                      app_id='upgrades', severity='warning',
+                                      title=title, message=message, actions=[{
+                                          'type': 'dismiss'
+                                      }], group='admin')
+    elif 'started-dist-upgrade' in reason:
+        logger.info('Started dist upgrade.')
+        title = gettext_noop('Distribution update started')
+        message = gettext_noop(
+            'Started update to next stable release. This may take a long '
+            'time to complete.')
+        Notification.update_or_create(id='upgrades-dist-upgrade-started',
+                                      app_id='upgrades', severity='info',
+                                      title=title, message=message, actions=[{
+                                          'type': 'dismiss'
+                                      }], group='admin')
+    else:
+        logger.warning('Unhandled result of start-dist-upgrade: %s, %s',
+                       dist_upgrade_started, reason)
 
 
 def is_backports_requested():
@@ -263,7 +247,7 @@ def set_dist_upgrade_enabled(enabled=True):
 
 def is_backports_enabled():
     """Return whether backports are enabled in the system configuration."""
-    return os.path.exists(BACKPORTS_SOURCES_LIST)
+    return os.path.exists(privileged.BACKPORTS_SOURCES_LIST)
 
 
 def get_current_release():
@@ -303,3 +287,14 @@ def can_enable_dist_upgrade():
     """Return whether dist upgrade can be enabled."""
     release, _ = get_current_release()
     return release not in ['unstable', 'testing']
+
+
+def can_test_dist_upgrade():
+    """Return whether dist upgrade can be tested."""
+    return can_enable_dist_upgrade() and cfg.develop
+
+
+def test_dist_upgrade():
+    """Test dist-upgrade from stable to testing."""
+    if can_test_dist_upgrade():
+        try_start_dist_upgrade(test=True)
