@@ -1,23 +1,20 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure WordPress.
-"""
+"""FreedomBox app to configure WordPress."""
 
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import cfg, frontpage, menu
+from plinth.config import DropinConfigs
 from plinth.daemon import Daemon
 from plinth.modules.apache.components import Webserver
 from plinth.modules.backups.components import BackupRestore
 from plinth.modules.firewall.components import Firewall
 from plinth.package import Packages
+from plinth.privileged import service as service_privileged
 from plinth.utils import format_lazy
 
-from . import manifest
-
-PUBLIC_ACCESS_FILE = '/etc/wordpress/is_public'
+from . import manifest, privileged
 
 _description = [
     _('WordPress is a popular way to create and manage websites and blogs. '
@@ -46,7 +43,7 @@ class WordPressApp(app_module.App):
 
     app_id = 'wordpress'
 
-    _version = 2
+    _version = 4
 
     def __init__(self):
         """Create components for the app."""
@@ -85,9 +82,19 @@ class WordPressApp(app_module.App):
                 'php-zip',
                 # WordPress only supports MySQL/MariaDB as DB
                 'default-mysql-server',
-            ])
+            ],
+            conflicts=['libpam-tmpdir'],
+            conflicts_action=Packages.ConflictsAction.REMOVE)
         self.add(packages)
 
+        dropin_configs = DropinConfigs('dropin-configs-wordpress', [
+            '/etc/apache2/conf-available/wordpress-freedombox.conf',
+            '/etc/fail2ban/jail.d/wordpress-freedombox.conf',
+            '/etc/fail2ban/filter.d/wordpress-freedombox.conf',
+            '/etc/wordpress/freedombox-static.php',
+        ])
+        self.add(dropin_configs)
+
         firewall = Firewall('firewall-wordpress', info.name,
                             ports=['http', 'https'], is_external=True)
         self.add(firewall)
@@ -106,9 +113,17 @@ class WordPressApp(app_module.App):
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        actions.superuser_run('wordpress', ['setup'])
+        privileged.setup()
         if not old_version:
             self.enable()
+        elif old_version < 3:
+            # Apply changes to Apache configuration from v2 to v3.
+            service_privileged.reload('apache2')
+
+    def uninstall(self):
+        """De-configure and uninstall the app."""
+        super().uninstall()
+        privileged.uninstall()
 
 
 class WordPressBackupRestore(BackupRestore):
@@ -117,9 +132,9 @@ class WordPressBackupRestore(BackupRestore):
     def backup_pre(self, packet):
         """Save database contents."""
         super().backup_pre(packet)
-        actions.superuser_run('wordpress', ['dump-database'])
+        privileged.dump_database()
 
     def restore_post(self, packet):
         """Restore database contents."""
         super().restore_post(packet)
-        actions.superuser_run('wordpress', ['restore-database'])
+        privileged.restore_database()
