@@ -33,10 +33,6 @@ logger = logging.getLogger(__name__)
 
 base_url = config['DEFAULT']['url']
 
-_app_checkbox_id = {
-    'openvpn': 'id_openvpn-enabled',
-}
-
 # unlisted sites just use '/' + site_name as url
 _site_url = {
     'wiki': '/ikiwiki',
@@ -98,7 +94,7 @@ class _PageLoaded():
         is_stale = False
         try:
             self.element.has_class('whatever_class')
-        except StaleElementReferenceException:
+        except (StaleElementReferenceException, TypeError):
             # After a domain name change, Let's Encrypt will restart the web
             # server and could cause a connection failure.
             if driver.find_by_id('netErrorButtonContainer'):
@@ -398,17 +394,14 @@ def uninstall(browser, app_name):
 def _change_app_status(browser, app_name, change_status_to='enabled'):
     """Enable or disable application."""
     button = browser.find_by_css('button[name="app_enable_disable_button"]')
+    if not button:
+        raise RuntimeError('App enable/disable button not found')
 
-    if button:
-        should_enable_field = browser.find_by_id('id_should_enable')
-        if (should_enable_field.value == 'False' and change_status_to
-                == 'disabled') or (should_enable_field.value == 'True'
-                                   and change_status_to == 'enabled'):
-            submit(browser, element=button)
-    else:
-        checkbox_id = _app_checkbox_id[app_name]
-        change_checkbox_status(browser, app_name, checkbox_id,
-                               change_status_to)
+    should_enable_field = browser.find_by_id('id_should_enable')
+    if (should_enable_field.value == 'False' and change_status_to
+            == 'disabled') or (should_enable_field.value == 'True'
+                               and change_status_to == 'enabled'):
+        submit(browser, element=button)
 
 
 def app_enable(browser, app_name):
@@ -555,7 +548,10 @@ def backup_restore(browser, app_name, archive_name=None):
 def networks_set_firewall_zone(browser, zone):
     """"Set the network device firewall zone as internal or external."""
     nav_to_module(browser, 'networks')
+    # First active Ethernet connection
     device = browser.find_by_xpath(
+        '//span[contains(@class, "connection-type-label") and '
+        'contains(., "Ethernet") ]/../..'
         '//span[contains(@class, "badge-success") '
         'and contains(@class, "connection-status-label")]/following::a').first
     network_id = device['href'].split('/')[-3]
