@@ -9,8 +9,14 @@ from unittest.mock import Mock, call, patch
 import pytest
 
 from plinth.app import App
-from plinth.errors import ActionError, MissingPackageError
-from plinth.package import Package, Packages, packages_installed, remove
+from plinth.errors import MissingPackageError
+from plinth.package import Package, Packages, packages_installed
+
+
+@pytest.fixture(autouse=True)
+def fixture_clean_apps():
+    """Fixture to ensure clean set of global apps."""
+    App._all_apps = {}
 
 
 class TestPackageExpressions(unittest.TestCase):
@@ -54,7 +60,7 @@ def test_packages_init():
     assert component.possible_packages == ['foo', 'bar']
     assert component.component_id == 'test-component'
     assert not component.skip_recommends
-    assert component.conflicts is None
+    assert component.conflicts == []
     assert component.conflicts_action is None
 
     with pytest.raises(ValueError):
@@ -114,6 +120,110 @@ def test_packages_setup(install):
     install.assert_has_calls([call(['python3'], skip_recommends=False)])
 
 
+@patch('plinth.package.packages_installed')
+@patch('plinth.package.uninstall')
+@patch('plinth.package.install')
+def test_packages_setup_with_conflicts(install, uninstall, packages_installed):
+    """Test setting up packages with conflicts."""
+    packages_installed.return_value = ['exim4-base']
+
+    component = Packages('test-component', ['bash'], conflicts=['exim4-base'],
+                         conflicts_action=Packages.ConflictsAction.REMOVE)
+    component.setup(old_version=0)
+    uninstall.assert_has_calls([call(['exim4-base'], purge=False)])
+    install.assert_has_calls([call(['bash'], skip_recommends=False)])
+
+    uninstall.reset_mock()
+    install.reset_mock()
+    component = Packages('test-component', ['bash'], conflicts=['exim4-base'])
+    component.setup(old_version=0)
+    uninstall.assert_not_called()
+    install.assert_has_calls([call(['bash'], skip_recommends=False)])
+
+    uninstall.reset_mock()
+    install.reset_mock()
+    component = Packages('test-component', ['bash'],
+                         conflicts=['exim4-base', 'not-installed-package'],
+                         conflicts_action=Packages.ConflictsAction.IGNORE)
+    component.setup(old_version=0)
+    uninstall.assert_not_called()
+    install.assert_has_calls([call(['bash'], skip_recommends=False)])
+
+
+@patch('plinth.package.uninstall')
+def test_packages_uninstall(uninstall):
+    """Test uninstalling packages component."""
+
+    class TestApp(App):
+        """Test app"""
+        app_id = 'test-app'
+
+    component = Packages('test-component', ['python3', 'bash'])
+    app = TestApp()
+    app.add(component)
+    app.uninstall()
+    uninstall.assert_has_calls([call(['python3', 'bash'], purge=True)])
+
+
+@patch('plinth.package.uninstall')
+@patch('apt.Cache')
+def test_packages_uninstall_exclusion(cache, uninstall):
+    """Test excluding packages from other installed apps when uninstalling."""
+    cache.return_value = {
+        'package11': Mock(candidate=Mock(version='2.0', is_installed=True)),
+        'package12': Mock(candidate=Mock(version='3.0', is_installed=False)),
+        'package2': Mock(candidate=Mock(version='4.0', is_installed=True)),
+        'package3': Mock(candidate=Mock(version='5.0', is_installed=True)),
+    }
+
+    class TestApp1(App):
+        """Test app."""
+        app_id = 'test-app1'
+
+        def __init__(self):
+            super().__init__()
+            component = Packages('test-component11',
+                                 ['package11', 'package2', 'package3'])
+            self.add(component)
+
+            component = Packages('test-component12',
+                                 ['package12', 'package2', 'package3'])
+            self.add(component)
+
+    class TestApp2(App):
+        """Test app."""
+        app_id = 'test-app2'
+
+        def __init__(self):
+            super().__init__()
+            component = Packages('test-component2', ['package2'])
+            self.add(component)
+
+        def get_setup_state(self):
+            return App.SetupState.UP_TO_DATE
+
+    class TestApp3(App):
+        """Test app."""
+        app_id = 'test-app3'
+
+        def __init__(self):
+            super().__init__()
+            component = Packages('test-component3', ['package3'])
+            self.add(component)
+
+        def get_setup_state(self):
+            return App.SetupState.NEEDS_SETUP
+
+    app1 = TestApp1()
+    TestApp2()
+    TestApp3()
+    app1.uninstall()
+    uninstall.assert_has_calls([
+        call(['package11', 'package3'], purge=True),
+        call(['package12', 'package3'], purge=True)
+    ])
+
+
 @patch('apt.Cache')
 def test_diagnose(cache):
     """Test checking for latest version of the package."""
@@ -189,17 +299,3 @@ def test_packages_installed():
     assert len(packages_installed(())) == 0
     assert len(packages_installed(('unknown-package', ))) == 0
     assert len(packages_installed(('python3', ))) == 1
-
-
-@patch('plinth.actions.superuser_run')
-def test_remove(run):
-    """Test removing packages."""
-    remove(['package1', 'package2'])
-    run.assert_has_calls(
-        [call('packages', ['remove', '--packages', 'package1', 'package2'])])
-
-    run.reset_mock()
-    run.side_effect = ActionError()
-    remove(['package1'])
-    run.assert_has_calls(
-        [call('packages', ['remove', '--packages', 'package1'])])
