@@ -3,15 +3,18 @@
 Main FreedomBox views.
 """
 
+import datetime
 import time
 import urllib.parse
 
 from django.contrib import messages
 from django.core.exceptions import ImproperlyConfigured
 from django.http import Http404, HttpResponseBadRequest, HttpResponseRedirect
+from django.shortcuts import redirect
 from django.template.response import TemplateResponse
 from django.urls import reverse
 from django.utils.translation import gettext as _
+from django.views.decorators.http import require_POST
 from django.views.generic import TemplateView
 from django.views.generic.edit import FormView
 from stronghold.decorators import public
@@ -65,6 +68,15 @@ def _get_redirect_url_from_param(request):
     return reverse('index')
 
 
+def _has_backup_restore(app):
+    """Return whether an app implements backup/restore."""
+    from plinth.modules.backups.components import BackupRestore
+    return any([
+        component.has_data
+        for component in app.get_components_of_type(BackupRestore)
+    ])
+
+
 @public
 def index(request):
     """Serve the main index page."""
@@ -161,6 +173,15 @@ class AppView(FormView):
         super().__init__(*args, **kwargs)
         self._common_status = None
 
+    def dispatch(self, request, *args, **kwargs):
+        """If operations are running on the app, use a different view."""
+        operations = operation.manager.filter(self.app.app_id)
+        if operations:
+            view = AppOperationsView.as_view(app_id=self.app.app_id)
+            return view(request, *args, **kwargs)
+
+        return super().dispatch(request, *args, **kwargs)
+
     def post(self, request, *args, **kwargs):
         """Handle app enable/disable button separately."""
         if 'app_enable_disable_button' not in request.POST:
@@ -256,10 +277,41 @@ class AppView(FormView):
         context['has_diagnostics'] = self.app.has_diagnostics()
         context['port_forwarding_info'] = get_port_forwarding_info(self.app)
         context['app_enable_disable_form'] = self.get_enable_disable_form()
+        context['show_rerun_setup'] = True
+        context['show_uninstall'] = not self.app.info.is_essential
+        context['refresh_page_sec'] = None
 
         from plinth.modules.firewall.components import Firewall
         context['firewall'] = self.app.get_components_of_type(Firewall)
 
+        context['has_backup_restore'] = _has_backup_restore(self.app)
+
+        return context
+
+
+class AppOperationsView(TemplateView):
+    """View to show app page when some app operations are running."""
+    app_id = None  # Set when app is instantiated.
+    template_name = "app-operations.html"
+
+    @property
+    def app(self):
+        """Return the app for which this view is configured."""
+        if not self.app_id:
+            raise ImproperlyConfigured('Missing attribute: app_id')
+
+        return app_module.App.get(self.app_id)
+
+    def get_context_data(self, *args, **kwargs):
+        """Add additional context data for template."""
+        context = super().get_context_data(*args, **kwargs)
+        context['app_id'] = self.app.app_id
+        context['app_info'] = self.app.info
+        context['operations'] = operation.manager.filter(self.app.app_id)
+        context['refresh_page_sec'] = 0
+        if context['operations']:
+            context['refresh_page_sec'] = 3
+
         return context
 
 
@@ -273,6 +325,7 @@ class SetupView(TemplateView):
         app_id = self.kwargs['app_id']
         app = app_module.App.get(app_id)
 
+        context['app_id'] = app.app_id
         context['app_info'] = app.info
 
         # Report any installed conflicting packages that will be removed.
@@ -282,8 +335,13 @@ class SetupView(TemplateView):
         context['package_conflicts_action'] = package_conflicts_action
 
         # Reuse the value of setup_state throughout the view for consistency.
-        context['setup_state'] = app.get_setup_state()
+        setup_state = app.get_setup_state()
+        context['setup_state'] = setup_state
         context['operations'] = operation.manager.filter(app.app_id)
+        context['show_rerun_setup'] = False
+        context['show_uninstall'] = (
+            not app.info.is_essential
+            and setup_state != app_module.App.SetupState.NEEDS_SETUP)
 
         # Perform expensive operation only if needed.
         if not context['operations']:
@@ -345,6 +403,88 @@ class SetupView(TemplateView):
                    if component.has_unavailable_packages())
 
 
+@require_POST
+def rerun_setup_view(request, app_id):
+    """Re-run setup on an app.
+
+    This should be safe to perform on an already setup/running app. This may be
+    useful in situations where the app is broken for unknown reason as notified
+    by the diagnostics tests.
+    """
+    # Start the application setup, and refresh the page every few seconds to
+    # keep displaying the status.
+    setup.run_setup_on_app(app_id, rerun=True)
+
+    # Give a moment for the setup process to start and show meaningful status.
+    time.sleep(1)
+
+    return redirect(reverse(f'{app_id}:index'))
+
+
+class UninstallView(FormView):
+    """View to uninstall apps."""
+
+    form_class = forms.UninstallForm
+    template_name = 'uninstall.html'
+
+    def __init__(self, **kwargs):
+        """Initialize extra instance members."""
+        super().__init__(**kwargs)
+        self.app = None
+
+    def setup(self, request, *args, **kwargs):
+        """Initialize attributes shared by all view methods."""
+        super().setup(request, *args, **kwargs)
+        app_id = self.kwargs['app_id']
+        self.app = app_module.App.get(app_id)
+        self.has_backup_restore = _has_backup_restore(self.app)
+
+    def dispatch(self, request, *args, **kwargs):
+        """Don't allow the view to be used on essential apps."""
+        if self.app.info.is_essential:
+            raise Http404
+
+        return super().dispatch(request, *args, **kwargs)
+
+    def get_form_kwargs(self):
+        """Return the keyword arguments for instantiating the form."""
+        kwargs = super().get_form_kwargs()
+        kwargs['has_backup_restore'] = self.has_backup_restore
+        return kwargs
+
+    def get_context_data(self, *args, **kwargs):
+        """Add app information to the context data."""
+        context = super().get_context_data(*args, **kwargs)
+        context['app_info'] = self.app.info
+        return context
+
+    def get_success_url(self):
+        """Return the URL to redirect to after uninstall."""
+        return reverse(self.kwargs['app_id'] + ':index')
+
+    def form_valid(self, form):
+        """Uninstall the app."""
+        # Backup the app
+        if self.has_backup_restore and form.cleaned_data['should_backup']:
+            repository_id = form.cleaned_data['repository']
+
+            import plinth.modules.backups.repository as repository_module
+            repository = repository_module.get_instance(repository_id)
+            if repository.flags.get('mountable'):
+                repository.mount()
+
+            name = datetime.datetime.now().strftime(
+                '%Y-%m-%d:%H:%M:%S') + ' ' + str(
+                    _('before uninstall of {app_id}')).format(
+                        app_id=self.app.app_id)
+            repository.create_archive(name, [self.app.app_id])
+
+        # Uninstall
+        setup.run_uninstall_on_app(self.app.app_id)
+
+        return super().form_valid(form)
+
+
 def notification_dismiss(request, id):
     """Dismiss a notification."""
     from .notification import Notification
